c Main program for solving Shell Dual Problem 
c by use of the exact penalty function method.
c The penalty coefficient is chosen automatically
c by the solver
        implicit none
        double precision x[allocatable](:), options(13), f
        integer n, nh, ng
        logical flg,flfc,flgc
        external dualshel,dualobjf,dualobjg,dualcntf,dualcntg

c All flags are set to 1:
          flg=.true.
          flfc=.true.
          flgc=.true.
c Allocate array X:          
         n=15
         allocate (x(n))
c Initialize the problem constants:
         call dualshel(x)
         nh=0
         ng=20
c Use the default optional parameters:
         call soptions(options)
         options(5)=1.d0
c Call the solver:         
         call solvopt(n,x,f,dualobjf,flg,dualobjg,options,
     1    flfc,dualcntf,flgc,dualcntg)
c Display the results:
      print '(//5x,a)','Function Value ===== Evaluations == Iterations'
      print '(/ 1x,g22.15,2x,i5,2h +,i5,5x,i5)',
     1   f,idnint(options(10)),idnint(options(11)),idnint(options(9))
      print '(//1x,16hOptimum Point X:,//((1x,g22.15)))', x

        deallocate (x)
        end

c  Initialization routine:
      subroutine dualshel(x)
      implicit none
      double precision f,fc,s
      double precision x(*), g(*), gc(*)
      double precision a(10,5), b(10), c(5,5), d(5), e(5)
      integer im, i, j
      save a,b,c,d,e,im
      data a /
     c  -16.d0,   0.d0,-3.5d0,   0.d0,   0.d0,   
     c    2.d0,  -1.d0, -1.d0,   1.d0,   1.d0,
     c    2.d0,  -2.d0,  0.d0,  -2.d0,  -9.d0,
     c    0.d0,  -1.d0, -2.d0,   2.d0,   1.d0,
     c    0.d0,   0.d0,  2.d0,   0.d0,  -2.d0,
     c   -4.d0,  -1.d0, -3.d0,   3.d0,   1.d0,
     c    1.d0,   .4d0,  0.d0,  -4.d0,   1.d0,
     c    0.d0,  -1.d0, -2.d0,   4.d0,   1.d0,
     c    0.d0,   2.d0,  0.d0,  -1.d0,  -2.8d0,
     c    0.d0,  -1.d0, -1.d0,   5.d0,   1.d0/
      data   b / -40.d0, -2.d0, -.25d0, -4.d0, -4.d0, -1.d0, -40.d0, 
     c        -60.d0, 5.d0, 1.d0 / 
      data   c /  30.d0, -20.d0, -10.d0,  32.d0, -10.d0,
     c        -20.d0,  39.d0,  -6.d0, -31.d0,  32.d0,
     c        -10.d0,  -6.d0,  10.d0,  -6.d0, -10.d0,
     c         32.d0, -31.d0,  -6.d0,  39.d0, -20.d0,
     c        -10.d0,  32.d0, -10.d0, -20.d0,  30.d0 / 
      data d /  4.d0, 8.d0, 10.d0, 6.d0, 2.d0 / 
      data e /  -15.d0, -27.d0, -36.d0, -18.d0, -12.d0 / 
c Standard starting point:
      do i=1,15
       x(i)=1.d-4
      enddo
      x(12)=6.d1 
      return
      
c Objective function
      entry dualobjf(x,f)

      f=0.d0
      do i=1,5
         do j=1,5
            f=f+c(i,j)*x(i)*x(j)
         enddo
         f=f+d(i)*x(i)**3*2.d0
      enddo
      do i=6,15
         f=f-b(i-5)*x(i)
      enddo
      return
      
c Gradient of the objective function
      entry dualobjg(x,g)

      do i=1,15
         if(i.le.5) then
           g(i)=6.d0*d(i)*x(i)**2
           s=0.d0
           do j=1,5
             s=s+c(i,j)*x(j)
           enddo
           g(i)=g(i)+2.d0*s
         else
           g(i)=-b(i-5)
         endif  
      enddo
      return

c The MAXIMAL RESIDIAL for the set of all constraints:
      entry dualcntf(x,fc)

      fc=0.d0
      im=0
      do i=1,20
        if(i.le.5) then
c The first 5 nonlinear inequalities:
          s=0.d0
          do  j=1,10
             s=s+a(j,i)*x(j+5)
          enddo
          do  j=1,5
             s=s-2.d0*c(j,i)*x(j)
          enddo
          s=s-e(i)-3.d0*d(i)*x(i)**2
        else
c Nonnegative values:
          s=-x(i-5)
        endif
        if (s.gt.fc) then
          fc=s
          im=i
        endif    
      enddo    
      return
      
c Gradient of the constraint with the MAXIMAL RESIDIAL:
      entry dualcntg(x,gc)

      do j=1,15
        gc(j)=0.d0
      enddo  
      if (im.gt.0) then
        if(im.le.5) then
c The first 5 nonlinear inequalities:
          do j=1,15
            if(j .le. 5)  then
                gc(j)=-2.d0*c(j,im)
            else    
                gc(j)=a(j-5,im)
            endif    
          enddo
          gc(im)=gc(im)-6.d0*d(im)*x(im)
        else
c Nonnegative values:
          gc(im-5)=-1.d0
        endif
      endif
      return

      end
