c Main program for solving Shell Dual Problem 
c by use of the exact penalty function method.
c The penalty coefficient is chosen automatically
c by the solver.
c This is an example on calculating gradients by
c forward differences.
        implicit none
        double precision x[allocatable](:), options(13), f
        integer n, nh, ng, i
        logical flg,flfc,flgc
        external dualshel,dualobjf,dualcntf,null
        character*11 name

      open (3,file='c:\fsources\solvopt\dual.res',access='append')
      open (4,file='c:\fsources\solvopt\dual.tbl',access='append')

c Analytical gradients of the objective function are not supplied:
          flg=.false.
c Constrained problem:
          flfc=.true.
c Analytical gradients of the constraints are not supplied:
          flgc=.false.
c Allocate array X:          
         n=15
         allocate (x(n))
         nh=0
         ng=20
         name='DS CNT APPR'
c Initialize the problem constants:
         call dualshel(x)
c Use the default optional parameters:
         call soptions(options)
c Call the solver:         
         call solvopt(n,x,f,dualobjf,flg,null,options,
     1    flfc,dualcntf,flgc,null)
c Display the results:
      print '(//5x,a)','Function Value ===== Evaluations == Iterations'
      print '(/ 1x,g22.15,2x,i5,2h +,i5,5x,i5)',
     1   f,idnint(options(10)),idnint(options(11)),idnint(options(9))
      print '(//1x,16hOptimum Point X:,//((1x,g22.15)))', x

      write(4,'(1x,a11,3(1h|,i5),1h|,d16.9,4(1h|,i6))')
     1 name,
     2 N,NH,NG,F,idnint(options(10)),idnint(options(11)),
     3 idnint(options(12)),idnint(options(13))

      write(3,'(//1x,a,a)')
     1 'PROBLEM ID = ',name
      write(3,'(/5x,a)')
     1 'Function Value ===== Evaluations == Iterations'
      write(3,'(/ 1x,g22.15,2x,i5,2h +,i5,5x,i5)')
     1   f,idnint(options(10)),idnint(options(11)),idnint(options(9))
      if (nh+ng.gt.0) then
        write(3,'(/ 25x,i5,2h +,i5)')
     1   idnint(options(12)),idnint(options(13))
      endif
      write(3,'(//1x,16hOptimum Point X:,//((1x,g22.15)))') 
     1  (x(i),i=1,n)

        close (3,status='keep')
        close (4,status='keep')

        deallocate (x)
        end

c  Initialization routine:
      subroutine dualshel(x)
      implicit none
      double precision f,fc,s
      double precision x(*)
      double precision a(10,5), b(10), c(5,5), d(5), e(5)
      integer im, i, j
      save a,b,c,d,e
      data a /
     c  -16.d0,   0.d0,-3.5d0,   0.d0,   0.d0,   
     c    2.d0,  -1.d0, -1.d0,   1.d0,   1.d0,
     c    2.d0,  -2.d0,  0.d0,  -2.d0,  -9.d0,
     c    0.d0,  -1.d0, -2.d0,   2.d0,   1.d0,
     c    0.d0,   0.d0,  2.d0,   0.d0,  -2.d0,
     c   -4.d0,  -1.d0, -3.d0,   3.d0,   1.d0,
     c    1.d0,   .4d0,  0.d0,  -4.d0,   1.d0,
     c    0.d0,  -1.d0, -2.d0,   4.d0,   1.d0,
     c    0.d0,   2.d0,  0.d0,  -1.d0,  -2.8d0,
     c    0.d0,  -1.d0, -1.d0,   5.d0,   1.d0/
      data   b / -40.d0, -2.d0, -.25d0, -4.d0, -4.d0, -1.d0, -40.d0, 
     c        -60.d0, 5.d0, 1.d0 / 
      data   c /  30.d0, -20.d0, -10.d0,  32.d0, -10.d0,
     c        -20.d0,  39.d0,  -6.d0, -31.d0,  32.d0,
     c        -10.d0,  -6.d0,  10.d0,  -6.d0, -10.d0,
     c         32.d0, -31.d0,  -6.d0,  39.d0, -20.d0,
     c        -10.d0,  32.d0, -10.d0, -20.d0,  30.d0 / 
      data d /  4.d0, 8.d0, 10.d0, 6.d0, 2.d0 / 
      data e /  -15.d0, -27.d0, -36.d0, -18.d0, -12.d0 / 
c Standard starting point:
      do i=1,15
       x(i)=1.d-4
      enddo
      x(12)=6.d1 
      return
      
c Objective function
      entry dualobjf(x,f)

      f=0.d0
      do i=1,5
         do j=1,5
            f=f+c(i,j)*x(i)*x(j)
         enddo
         f=f+d(i)*x(i)**3*2.d0
      enddo
      do i=6,15
         f=f-b(i-5)*x(i)
      enddo
      return
      
c The MAXIMAL RESIDIAL for the set of all constraints:
      entry dualcntf(x,fc)

      fc=0.d0
      im=0
      do i=1,20
        if(i.le.5) then
c The first 5 nonlinear inequalities:
          s=0.d0
          do  j=1,10
             s=s+a(j,i)*x(j+5)
          enddo
          do  j=1,5
             s=s-2.d0*c(j,i)*x(j)
          enddo
          s=s-e(i)-3.d0*d(i)*x(i)**2
        else
c Nonnegative values:
          s=-x(i-5)
        endif
        if (s.gt.fc) then
          fc=s
          im=i
        endif    
      enddo    
      return
      
      end
